﻿using Microsoft.Extensions.Logging;
using DinerContracts.BindingModels;
using DinerContracts.BusinessLogicsContracts;
using DinerContracts.ViewModels;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace Diner
{
    public partial class FormCreateSupply : Form
    {
        private readonly ILogger _logger;
        private readonly ISnackLogic _logicP;
        private readonly IShopLogic _logicS;
        private List<ShopViewModel> _shopList = new List<ShopViewModel>();
        private List<SnackViewModel> _snackList = new List<SnackViewModel>();

        public FormCreateSupply(ILogger<FormCreateSupply> logger, ISnackLogic logicP, IShopLogic logicS)
        {
            InitializeComponent();
            _logger = logger;
            _logicP = logicP;
            _logicS = logicS;
        }

        private void FormCreateSupply_Load(object sender, EventArgs e)
        {
            _shopList = _logicS.ReadList(null);
            _snackList = _logicP.ReadList(null);
            if (_shopList != null)
            {
                comboBoxShop.DisplayMember = "ShopName";
                comboBoxShop.ValueMember = "Id";
                comboBoxShop.DataSource = _shopList;
                comboBoxShop.SelectedItem = null;
                _logger.LogInformation("Загрузка магазинов для поставок");
            }
            if (_snackList != null)
            {
                comboBoxSnack.DisplayMember = "SnackName";
                comboBoxSnack.ValueMember = "Id";
                comboBoxSnack.DataSource = _snackList;
                comboBoxSnack.SelectedItem = null;
                _logger.LogInformation("Загрузка закуски для поставок");
            }
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (comboBoxShop.SelectedValue == null)
            {
                MessageBox.Show("Выберите магазин", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxSnack.SelectedValue == null)
            {
                MessageBox.Show("Выберите закуску", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Создание поставки");
            try
            {
                var operationResult = _logicS.MakeSupply(new SupplyBindingModel
                {
                    ShopId = Convert.ToInt32(comboBoxShop.SelectedValue),
                    SnackId = Convert.ToInt32(comboBoxSnack.SelectedValue),
                    Count = Convert.ToInt32(textBoxCount.Text)
                });
                if (!operationResult)
                {
                    throw new Exception("Ошибка при создании поставки. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания поставки");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
