﻿using Microsoft.Extensions.Logging;
using DinerContracts.BusinessLogicsContracts;
using DinerContracts.SearchModels;
using DinerContracts.StoragesContracts;
using DinerContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace DinerView
{
    public partial class FormSellSnack : Form
    {
        private readonly ILogger _logger;
        private readonly ISnackLogic _logicP;
        private readonly IShopLogic _logicS;
        private List<SnackViewModel> _snackList = new List<SnackViewModel>();
        public FormSellSnack(ILogger<FormSellSnack> logger, ISnackLogic logicP, IShopLogic logicS)
        {
            InitializeComponent();
            _logger = logger;
            _logicP = logicP;
            _logicS = logicS;
        }
        private void FormSellingPizza_Load(object sender, EventArgs e)
        {
            _snackList = _logicP.ReadList(null);
            if (_snackList != null)
            {
                comboBoxSnack.DisplayMember = "SnackName";
                comboBoxSnack.ValueMember = "Id";
                comboBoxSnack.DataSource = _snackList;
                comboBoxSnack.SelectedItem = null;
                _logger.LogInformation("Загрузка закуски для продажи");
            }
        }
        private void ButtonSell_Click(object sender, EventArgs e)
        {
            if (comboBoxSnack.SelectedValue == null)
            {
                MessageBox.Show("Выберите закуску", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Создание покупки");
            try
            {
                bool resout = _logicS.Sale(new SupplySearchModel
                {
                    SnackId = Convert.ToInt32(comboBoxSnack.SelectedValue),
                    Count = Convert.ToInt32(textBoxCount.Text)
                });
                if (resout)
                {
                    _logger.LogInformation("Проверка пройдена, продажа  проведена");
                    MessageBox.Show("Продажа проведена", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    DialogResult = DialogResult.OK;
                    Close();
                }
                else {
                    _logger.LogInformation("Проверка не пройдена");
                    MessageBox.Show("Продажа не может быть создана.", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            catch(Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания покупки");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }
        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
