using DinerContracts.BusinessLogicsContracts;
using DinerContracts.StoragesContracts;
using DinerDatabaseImplement.Implements;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using DinerBusinessLogic.BusinessLogics;
using DinerBusinessLogic.Implements;
using DinerBusinessLogic.OfficePackage;

using DinerView;

namespace Diner
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();
            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }
        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IComponentStorage, ComponentStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<ISnackStorage, SnackStorage>();
            services.AddTransient<IShopStorage, ShopStorage>();

            services.AddTransient<IComponentLogic, ComponentLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<ISnackLogic, SnackLogic>();
            services.AddTransient<IReportLogic, ReportLogic>();
            services.AddTransient<IShopLogic, ShopLogic>();

            services.AddTransient<AbstractSaveToWord, SaveToWord>();
            services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            services.AddTransient<AbstractSaveToPdf, SaveToPdf>();

            services.AddTransient<FormComponent>();
            services.AddTransient<FormComponents>();
            services.AddTransient<FormCreateOrder>();
            services.AddTransient<FormSnack>();
            services.AddTransient<FormSnackComponent>();
            services.AddTransient<FormSnacks>();
            services.AddTransient<FormShop>();
            services.AddTransient<FormShops>();
            services.AddTransient<FormCreateSupply>();
            services.AddTransient<FormSellSnack>();
            services.AddTransient<FormMain>();
            services.AddTransient<FormReportSnackComponents>();
            services.AddTransient<FormReportOrders>();
            services.AddTransient<FormReportShop>();
            services.AddTransient<FormReportGroupedOrders>();
        }
    }
}