﻿using DinerContracts.BindingModels;
using DinerContracts.ViewModels;
using DinerDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DinerDatabaseImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; set; }
        public string ShopName { get; set; } = String.Empty;

        public string Adress { get; set; } = String.Empty;

        public DateTime OpeningDate { get; set; }

        public int SnackMaxCount { get; set; }

        private Dictionary<int, (ISnackModel, int)>? _shopSnacks = null;
        public Dictionary<int, (ISnackModel, int)> ShopSnacks
        {
            get
            {
                if (_shopSnacks == null) {
                    if (_shopSnacks == null)
                    {
                        _shopSnacks = Snacks
                                .ToDictionary(recSP => recSP.SnackId, recSP => (recSP.Snack as ISnackModel, recSP.Count));
                    }
                    return _shopSnacks;
                }
                return _shopSnacks;
            }
        }

        [ForeignKey("ShopId")]
        public List<ShopSnacks> Snacks { get; set; } = new();

        public static Shop Create(DinerDatabase context, ShopBindingModel model)
        {
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Adress = model.Adress,
                OpeningDate = model.OpeningDate,
                Snacks = model.ShopSnacks.Select(x => new ShopSnacks
                {
                    Snack = context.Snacks.First(y => y.Id == x.Key),
                    Count = x.Value.Item2
                }).ToList(),
				SnackMaxCount = model.SnackMaxCount
			};
        }

        public void Update(ShopBindingModel model)
        {
            ShopName = model.ShopName;
            Adress = model.Adress;
            OpeningDate = model.OpeningDate;
			SnackMaxCount = model.SnackMaxCount;
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Adress = Adress,
            OpeningDate = OpeningDate,
            ShopSnacks = ShopSnacks,
            SnackMaxCount = SnackMaxCount
		};

        

        public void UpdateSnacks(DinerDatabase context, ShopBindingModel model)
        {
            var ShopSnacks = context.ShopSnacks.Where(rec => rec.ShopId == model.Id).ToList();
            if (ShopSnacks != null && this.ShopSnacks.Count > 0)
            {
                context.ShopSnacks.RemoveRange(ShopSnacks.Where(rec => !model.ShopSnacks.ContainsKey(rec.SnackId)));
                context.SaveChanges();
				ShopSnacks = context.ShopSnacks.Where(rec => rec.ShopId == model.Id).ToList();
				foreach (var updateSnack in ShopSnacks)
                {
                    updateSnack.Count = model.ShopSnacks[updateSnack.SnackId].Item2;
                    model.ShopSnacks.Remove(updateSnack.SnackId);
                }
                context.SaveChanges();
            }
            var shop = context.Shops.First(x => x.Id == Id);
            foreach (var ar in model.ShopSnacks)
            {
                context.ShopSnacks.Add(new ShopSnacks
                {
                    Shop = shop,
                    Snack = context.Snacks.First(x => x.Id == ar.Key),
                    Count = ar.Value.Item2
                });
                context.SaveChanges();
            }
            _shopSnacks = null;
        }

        public void SnacksDictionatyUpdate(DinerDatabase context)
        {
            UpdateSnacks(context, new ShopBindingModel
            {
                Id = Id,
				ShopSnacks = ShopSnacks,
            });
        }

    }
}
