﻿using DinerContracts.BindingModels;
using DinerContracts.SearchModels;
using DinerContracts.StoragesContracts;
using DinerContracts.ViewModels;
using DinerFileImplement.Models;

namespace DinerFileImplement.Implements
{
    public class SnackStorage : ISnackStorage
    {
        private readonly DataFileSingleton source;
        public SnackStorage()
        {
            source = DataFileSingleton.GetInstance();
        }
        public List<SnackViewModel> GetFullList()
        {
            return source.Snacks.Select(x => x.GetViewModel).ToList();
        }
        public List<SnackViewModel> GetFilteredList(SnackSearchModel model)
        {
            if (string.IsNullOrEmpty(model.SnackName))
            {
                return new();
            }
            return source.Snacks.Where(x => x.SnackName.Contains(model.SnackName)).Select(x => x.GetViewModel).ToList();
        }
        public SnackViewModel? GetElement(SnackSearchModel model)
        {
            if (string.IsNullOrEmpty(model.SnackName) && !model.Id.HasValue)
            {
                return null;
            }
            return source.Snacks.FirstOrDefault(x =>
            (!string.IsNullOrEmpty(model.SnackName) && x.SnackName == model.SnackName) ||
            (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }
        public SnackViewModel? Insert(SnackBindingModel model)
        {
            model.Id = source.Snacks.Count > 0 ? source.Snacks.Max(x => x.Id) + 1 : 1;
            var newSnack = Snack.Create(model);
            if (newSnack == null)
            {
                return null;
            }
            source.Snacks.Add(newSnack);
            source.SaveSnacks();
            return newSnack.GetViewModel;
        }
        public SnackViewModel? Update(SnackBindingModel model)
        {
            var Snack = source.Snacks.FirstOrDefault(x => x.Id == model.Id);
            if (Snack == null)
            {
                return null;
            }
            Snack.Update(model);
            source.SaveSnacks();
            return Snack.GetViewModel;
        }
        public SnackViewModel? Delete(SnackBindingModel model)
        {
            var Snack = source.Snacks.FirstOrDefault(x => x.Id == model.Id);
            if (Snack != null)
            {
                source.Snacks.Remove(Snack);
                source.SaveSnacks();
                return Snack.GetViewModel;
            }
            return null;
        }
    }
}
