﻿using DinerDataModels.Models;
using DinerContracts.BindingModels;
using DinerContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace DinerFileImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        public string ShopName { get; private set; } = string.Empty;
        public string Adress { get; private set; } = string.Empty;
        public DateTime OpeningDate { get; private set; }
        public Dictionary<int, int> Snacks { get; private set; } = new();
        private Dictionary<int, (ISnackModel, int)>? _shopSnacks = null;
        public Dictionary<int, (ISnackModel, int)> ShopSnacks {
            get {
                if (_shopSnacks == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _shopSnacks = Snacks.ToDictionary(x => x.Key, y => ((source.Snacks.FirstOrDefault(z => z.Id == y.Key) as ISnackModel)!, y.Value));
                }
                return _shopSnacks;
            }
        }
        public int SnackMaxCount { get; private set; }
        public static Shop? Create(ShopBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Adress = model.Adress,
                OpeningDate = model.OpeningDate,
                Snacks = model.ShopSnacks.ToDictionary(x => x.Key, x => x.Value.Item2),
                SnackMaxCount = model.SnackMaxCount
            };
        }
        public static Shop? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ShopName = element.Element("ShopName")!.Value,
                Adress = element.Element("Adress")!.Value,
                OpeningDate = Convert.ToDateTime(element.Element("OpeningDate")!.Value),
                Snacks = element.Element("ShopSnacks")!.Elements("ShopSnack")!.ToDictionary(x => Convert.ToInt32(x.Element("Key")?.Value),
                x => Convert.ToInt32(x.Element("Value")?.Value)),
                SnackMaxCount = Convert.ToInt32(element.Element("SnackMaxCount")!.Value)
            };
        }
        public void Update(ShopBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            Adress = model.Adress;
            OpeningDate = model.OpeningDate;
            SnackMaxCount = model.SnackMaxCount;
            Snacks = model.ShopSnacks.ToDictionary(x => x.Key, x => x.Value.Item2);
            _shopSnacks = null;
        }
        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Adress = Adress,
            OpeningDate = OpeningDate,
            ShopSnacks = ShopSnacks,
            SnackMaxCount = SnackMaxCount
        };

        public XElement GetXElement => new("Shop",
            new XAttribute("Id", Id),
            new XElement("ShopName",ShopName),
            new XElement("Adress", Adress),
            new XElement("OpeningDate", OpeningDate.ToString()),
            new XElement("ShopSnacks", Snacks.Select(
                x => new XElement("ShopSnack", new XElement("Key",x.Key),new XElement("Value",x.Value))).ToArray()),
            new XElement("SnackMaxCount", SnackMaxCount.ToString())
            );

        public void SnackUpdate() {
            _shopSnacks = null;
        }
    }
}
